@@ -2,6 +2,7 @@
  */
 #ifndef eslFILEPARSER_INCLUDED
 #define eslFILEPARSER_INCLUDED
+#include "esl_config.h"
 
 #include <stdio.h>
 #include "easel.h"
@@ -18,14 +19,14 @@ typedef struct {
   char  errbuf[eslERRBUFSIZE];  /* for holding error diagnostics                   */
 
   int   is_buffer;              /* the file has been buffered into memory          */
-  char *mem_buffer;             /* pointer to the buffered file                    */
+  const char *mem_buffer;       /* pointer to the buffered file                    */
   int   mem_size;               /* size of the buffered file                       */
   int   mem_pos;                /* current position in the buffer                  */
 } ESL_FILEPARSER;
 
 extern int  esl_fileparser_Open(const char *filename, const char *envvar, ESL_FILEPARSER **ret_efp);
 extern ESL_FILEPARSER *esl_fileparser_Create(FILE *fp);
-extern ESL_FILEPARSER *esl_fileparser_CreateMapped(void *buffer, int size);
+extern ESL_FILEPARSER *esl_fileparser_CreateMapped(const void *buffer, int size);
 extern int  esl_fileparser_SetCommentChar  (ESL_FILEPARSER *efp, char c);
 extern int  esl_fileparser_GetToken        (ESL_FILEPARSER *efp, char **opt_tok, int *opt_toklen);
 extern int  esl_fileparser_NextLine        (ESL_FILEPARSER *efp);
@@ -36,12 +37,4 @@ extern void esl_fileparser_Destroy         (ESL_FILEPARSER *efp);
 extern void esl_fileparser_Close           (ESL_FILEPARSER *efp);
 
 #endif /*eslFILEPARSER_INCLUDED */
-/*****************************************************************
- * Easel - a library of C functions for biological sequence analysis
- * Version 0.43; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute
- * Other copyrights also apply. See the LICENSE file for a full list.
- * 
- * Easel is open source software, distributed under the BSD license. See
- * the LICENSE file for more details.
- *****************************************************************/
+
