@@ -18,9 +18,10 @@
 
 #include "easel.h"
 #include "esl_stats.h"
-#include "esl_histogram.h"
 #include "esl_vectorops.h"
 
+#include "esl_histogram.h"
+
 static int esl_histogram_sort(ESL_HISTOGRAM *h);
 
 
@@ -1026,6 +1027,12 @@ esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
  * Purpose:   Print observed (and expected, if set) binned counts
  *            in a histogram <h> to open file pointer <fp>
  *            in xmgrace XY input file format.
+ *            
+ *            The number that's plotted on the X axis is the minimum
+ *            (starting) value of the bin's interval. The Y value is
+ *            the total number of counts in the interval (x,x+w] for bin
+ *            width w. In xmgrace, you want to set "right stairs" as
+ *            the line type in an XY plot.
  *
  * Returns:   <eslOK> on success.
  *
@@ -1045,7 +1052,10 @@ esl_histogram_Plot(FILE *fp, ESL_HISTOGRAM *h)
       x = esl_histogram_Bin2LBound(h,i);
       if (fprintf(fp, "%f %llu\n", x, (unsigned long long) h->obs[i]) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
     }
-  if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
+  x = esl_histogram_Bin2LBound(h,i);   /* Print a trailing y=0, needed to make xmgrace display the last bar */
+  if (fprintf(fp, "%f %d\n", x, 0) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
+  if (fprintf(fp, "&\n")           < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
+
 
   /* Second data set is the theoretical (expected) histogram
    */
@@ -2060,17 +2070,5 @@ main(int argc, char **argv)
 /*::cexcerpt::histogram_example5::end::*/
 #endif /*eslHISTOGRAM_EXAMPLE5*/
 
-/*****************************************************************
- * Easel - a library of C functions for biological sequence analysis
- * Version 0.43; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute
- * Other copyrights also apply. See the LICENSE file for a full list.
- * 
- * Easel is open source software, distributed under the BSD license. See
- * the LICENSE file for more details.
- *
- * SVN $Id$
- * SVN $URL$
- *****************************************************************/
 
 
