@@ -1,7 +1,4 @@
 /* Support for MPI parallelization.
- * 
- * Only available when the entire Easel library is in use (HAVE_MPI
- * and eslLIBRARY are defined).
  * 
  * Contents:
  *    1. Communicating optional arrays.
@@ -11,10 +8,9 @@
  *    5. Unit tests.
  *    6. Test driver.
  *    7. Example.
- *    8. Copyright and license information.
  */
 #include "esl_config.h"		
-#if defined(HAVE_MPI) && defined(eslLIBRARY)
+#if defined(HAVE_MPI)
 #include <string.h>
 #include "mpi.h"
 
@@ -1217,7 +1213,7 @@ main(int argc, char **argv)
 
 
 
-#else /*!(HAVE_MPI && eslLIBRARY)*/
+#else /*!HAVE_MPI*/
 
 /* If we don't have MPI compiled in, provide some nothingness to:
  *   a. prevent Mac OS/X ranlib from bitching about .o file that "has no symbols" 
@@ -1231,14 +1227,5 @@ void esl_mpi_DoAbsolutelyNothing(void) { return; }
 int main(void) { return 0; }
 #endif
 
-#endif /*HAVE_MPI && eslLIBRARY*/
+#endif /*HAVE_MPI*/
 
-/*****************************************************************
- * Easel - a library of C functions for biological sequence analysis
- * Version 0.43; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute
- * Other copyrights also apply. See the LICENSE file for a full list.
- * 
- * Easel is open source software, distributed under the BSD license. See
- * the LICENSE file for more details.
- *****************************************************************/
