@@ -9,22 +9,17 @@
  */
 #ifndef eslSQIO_INCLUDED
 #define eslSQIO_INCLUDED
+#include "esl_config.h"
 
 #include <stdio.h>
 
-#include "esl_sqio_ascii.h"
-#ifdef eslAUGMENT_NCBI
-#include "esl_sqio_ncbi.h"
-#endif
-#include "esl_sq.h"
-
-#ifdef eslAUGMENT_ALPHABET
+#include "easel.h"
 #include "esl_alphabet.h"
-#endif
-#ifdef eslAUGMENT_MSA
 #include "esl_msa.h"
 #include "esl_msafile.h"
-#endif
+#include "esl_sq.h"
+#include "esl_sqio_ascii.h"
+#include "esl_sqio_ncbi.h"
 
 /*::cexcerpt::sq_sqio_data::begin::*/
 /* ESL_SQDATA:
@@ -32,9 +27,7 @@
  */
 typedef union {
   ESL_SQASCII_DATA ascii;
-#ifdef eslAUGMENT_NCBI
   ESL_SQNCBI_DATA  ncbi;
-#endif
 } ESL_SQDATA;
 /*::cexcerpt::sq_sqio_data::end::*/
 
@@ -46,11 +39,7 @@ typedef struct esl_sqio_s {
 
   /* In digital mode, we have an alphabet ptr                             */
   int   do_digital;	      /* TRUE if we're reading in digital mode    */
-#if defined(eslAUGMENT_ALPHABET)  
   const ESL_ALPHABET *abc;
-#else
-  void               *abc;
-#endif
 
   /* Format-specific configuration                                        */
   int     format;	      /* Format code of this file                 */
@@ -69,9 +58,8 @@ typedef struct esl_sqio_s {
   int   (*read_window)     (struct esl_sqio_s *sqfp, int C, int W, ESL_SQ *sq);
   int   (*echo)            (struct esl_sqio_s *sqfp, const ESL_SQ *sq, FILE *ofp);
 
-  int   (*read_block)      (struct esl_sqio_s *sqfp, ESL_SQ_BLOCK *sqBlock, int max_residues, int max_sequences, int long_target);
+  int   (*read_block)      (struct esl_sqio_s *sqfp, ESL_SQ_BLOCK *sqBlock, int max_residues, int max_sequences, int max_init_window, int long_target);
 
-#ifdef eslAUGMENT_SSI
   int   (*open_ssi)        (struct esl_sqio_s *sqfp, const char *ssifile_hint);
   int   (*pos_by_key)      (struct esl_sqio_s *sqfp, const char *key);
   int   (*pos_by_number)   (struct esl_sqio_s *sqfp, int which);
@@ -79,7 +67,6 @@ typedef struct esl_sqio_s {
   int   (*fetch)           (struct esl_sqio_s *sqfp, const char *key, ESL_SQ *sq);
   int   (*fetch_info)      (struct esl_sqio_s *sqfp, const char *key, ESL_SQ *sq);
   int   (*fetch_subseq)    (struct esl_sqio_s *sqfp, const char *source, int64_t start, int64_t end, ESL_SQ *sq);
-#endif
 
   int   (*is_rewindable)   (const struct esl_sqio_s *sqfp);
   const char *(*get_error) (const struct esl_sqio_s *sqfp);
@@ -87,7 +74,6 @@ typedef struct esl_sqio_s {
   ESL_SQDATA data;            /* format specific data                     */
 } ESL_SQFILE;
 
-#if defined(eslAUGMENT_ALPHABET)  
 /* ESL_SQCACHE:
  * A entire database cached into memory.
  */
@@ -109,7 +95,6 @@ typedef struct esl_sqcache_s {
   uint64_t            res_size;    /* size of residue memory allocation           */
   uint64_t            hdr_size;    /* size of header memory allocation            */
 } ESL_SQCACHE;
-#endif
 
 /*::cexcerpt::sq_sqio_format::begin::*/
 /* Unaligned file format codes
@@ -139,17 +124,15 @@ typedef struct esl_sqcache_s {
 extern int  esl_sqfile_Open(const char *seqfile, int fmt, const char *env, ESL_SQFILE **ret_sqfp);
 extern void esl_sqfile_Close(ESL_SQFILE *sqfp);
 
-#ifdef eslAUGMENT_ALPHABET
 extern int  esl_sqfile_OpenDigital(const ESL_ALPHABET *abc, const char *filename, int format, const char *env, ESL_SQFILE **ret_sqfp);
 extern int  esl_sqfile_SetDigital(ESL_SQFILE *sqfp, const ESL_ALPHABET *abc);
 extern int  esl_sqfile_GuessAlphabet(ESL_SQFILE *sqfp, int *ret_type);
-#endif
 
 extern int   esl_sqio_Read        (ESL_SQFILE *sqfp, ESL_SQ *sq);
 extern int   esl_sqio_ReadInfo    (ESL_SQFILE *sqfp, ESL_SQ *sq);
 extern int   esl_sqio_ReadWindow  (ESL_SQFILE *sqfp, int C, int W, ESL_SQ *sq);
 extern int   esl_sqio_ReadSequence(ESL_SQFILE *sqfp, ESL_SQ *sq);
-extern int   esl_sqio_ReadBlock   (ESL_SQFILE *sqfp, ESL_SQ_BLOCK *sqBlock, int max_residues, int max_sequences, int long_target);
+extern int   esl_sqio_ReadBlock   (ESL_SQFILE *sqfp, ESL_SQ_BLOCK *sqBlock, int max_residues, int max_sequences, int max_init_window, int long_target);
 extern int   esl_sqio_Parse       (char *buffer, int size, ESL_SQ *s, int format);
 
 extern int   esl_sqio_Write       (FILE *fp, ESL_SQ *s, int format, int update);
@@ -164,7 +147,6 @@ extern int   esl_sqfile_Position(ESL_SQFILE *sqfp, off_t offset);
 extern int   esl_sqio_Ignore(ESL_SQFILE *sqfp, const char *ignoredchars);
 extern int   esl_sqio_AcceptAs(ESL_SQFILE *sqfp, char *xchars, char readas);
 
-#ifdef eslAUGMENT_SSI
 extern int   esl_sqfile_OpenSSI         (ESL_SQFILE *sqfp, const char *ssifile_hint);
 extern int   esl_sqfile_PositionByKey   (ESL_SQFILE *sqfp, const char *key);
 extern int   esl_sqfile_PositionByNumber(ESL_SQFILE *sqfp, int which);
@@ -172,24 +154,9 @@ extern int   esl_sqfile_PositionByNumber(ESL_SQFILE *sqfp, int which);
 extern int   esl_sqio_Fetch      (ESL_SQFILE *sqfp, const char *key, ESL_SQ *sq);
 extern int   esl_sqio_FetchInfo  (ESL_SQFILE *sqfp, const char *key, ESL_SQ *sq);
 extern int   esl_sqio_FetchSubseq(ESL_SQFILE *sqfp, const char *source, int64_t start, int64_t end, ESL_SQ *sq);
-#endif
 
 extern int   esl_sqfile_Cache(const ESL_ALPHABET *abc, const char *seqfile, int fmt, const char *env, ESL_SQCACHE **ret_sqcache);
 extern void  esl_sqfile_Free(ESL_SQCACHE *sqcache);
 
-
-
-
-
-
-
 #endif /*eslSQIO_INCLUDED*/
-/*****************************************************************
- * Easel - a library of C functions for biological sequence analysis
- * Version 0.43; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute
- * Other copyrights also apply. See the LICENSE file for a full list.
- * 
- * Easel is open source software, distributed under the BSD license. See
- * the LICENSE file for more details.
- *****************************************************************/
+
