@@ -1,19 +1,14 @@
 /* Weibull distributions.
  * 
  * SRE, Tue Aug  9 10:48:35 2005 [St. Louis]
- * SVN $Id$
- * SVN $URL$
  */
 #ifndef eslWEIBULL_INCLUDED
 #define eslWEIBULL_INCLUDED
+#include "esl_config.h"
 
-#ifdef eslAUGMENT_RANDOM
-#include <esl_random.h>
-#endif
+#include "esl_histogram.h"
+#include "esl_random.h"
 
-#ifdef eslAUGMENT_HISTOGRAM
-#include <esl_histogram.h>
-#endif
 
 extern double esl_wei_pdf    (double x, double mu, double lambda, double tau);
 extern double esl_wei_logpdf (double x, double mu, double lambda, double tau);
@@ -33,27 +28,12 @@ extern int esl_wei_Plot(FILE *fp, double mu, double lambda, double tau,
 			double xmin, double xmax, double xstep);
 
 
-#ifdef eslAUGMENT_RANDOM
-extern double esl_wei_Sample(ESL_RANDOMNESS *r, double mu, double lambda, double tau);
-#endif
 
-#ifdef eslAUGMENT_MINIMIZER
+extern double esl_wei_Sample(ESL_RANDOMNESS *r, double mu, double lambda, double tau);
+
 extern int esl_wei_FitComplete(double *x, int n, double *ret_mu,
 			       double *ret_lambda, double *ret_tau);
-#ifdef eslAUGMENT_HISTOGRAM
 extern int esl_wei_FitCompleteBinned(ESL_HISTOGRAM *h, double *ret_mu,
 				     double *ret_lambda, double *ret_tau);
-#endif /*eslAUGMENT_HISTOGRAM*/
-#endif /*eslAUGMENT_MINIMIZER*/
-
 
 #endif /*eslWEIBULL_INCLUDED*/
-/*****************************************************************
- * Easel - a library of C functions for biological sequence analysis
- * Version 0.43; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute
- * Other copyrights also apply. See the LICENSE file for a full list.
- * 
- * Easel is open source software, distributed under the BSD license. See
- * the LICENSE file for more details.
- *****************************************************************/
