@@ -1,11 +1,11 @@
-#ifdef ESL_WITH_LAPACK
 /* interface_lapack.h
  * 
  * SRE, Tue Jul 13 15:11:51 2004 [St. Louis]
- * SVN $Id: interface_lapack.h 11 2005-01-06 11:44:17Z eddy $
  */
-#ifndef ESL_INTERFACE_LAPACK_INCLUDED
-#define ESL_INTERFACE_LAPACK_INCLUDED
+#ifndef eslINTERFACE_LAPACK_INCLUDED
+#define eslINTERFACE_LAPACK_INCLUDED
+#include "esl_config.h"
+#ifdef HAVE_LIBLAPACK
 
 /* This is the C interface to the Fortran77 dgeev routine,
  * provided by the LAPACK library:
@@ -19,5 +19,6 @@ extern void  dgeev_(char *jobvl, char *jobvr, int *n, double *a,
  */
 extern int esl_lapack_dgeev(ESL_DMATRIX *A, double **ret_Er, double **ret_Ei, ESL_DMATRIX **ret_VL, ESL_DMATRIX **ret_VR);
 
-#endif /*ESL_INTERFACE_LAPACK_INCLUDED*/
-#endif /*ESL_WITH_LAPACK*/
+#endif /*HAVE_LIBLAPACK*/
+#endif /*eslINTERFACE_LAPACK_INCLUDED*/
+
