@@ -209,7 +209,7 @@ create_ssi_index(ESL_GETOPTS *go, ESL_MSAFILE *afp)
     }
   
   if (esl_newssi_Write(ns) != eslOK) 
-    esl_fatal("Failed to write keys to ssi file %s\n", ssifile);
+    esl_fatal("\nFailed to write keys to ssi file %s:\n  %s", ssifile, ns->errbuf);
 
   printf("done.\n");
 
@@ -368,12 +368,3 @@ regurgitate_one_stockholm_entry(FILE *ofp, ESL_MSAFILE *afp)
 }
   
 
-/*****************************************************************
- * Easel - a library of C functions for biological sequence analysis
- * Version 0.43; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute
- * Other copyrights also apply. See the LICENSE file for a full list.
- * 
- * Easel is open source software, distributed under the BSD license. See
- * the LICENSE file for more details.
- *****************************************************************/
