@@ -6,11 +6,6 @@
  *   2. Interval / range computation
  *   3. Functions related to the original sequence
  *   4. FM data initialization, configuration, and reading from file
- *
- *  # 5. Unit tests.
- *  # 6. Test driver.
- *  # 7. Copyright and license.
- *
  */
 #include "p7_config.h"
 
@@ -143,8 +138,8 @@ fm_updateIntervalForward( const FM_DATA *fm, const FM_CFG *cfg, char c, FM_INTER
   interval_f->lower += (occLT_u - occLT_l);
   interval_f->upper = interval_f->lower + (occ_u - occ_l) - 1;
 
-  interval_bk->lower = abs(fm->C[(int)c]) + occ_l;
-  interval_bk->upper = abs(fm->C[(int)c]) + occ_u - 1;
+  interval_bk->lower = abs((int)(fm->C[(int)c])) + occ_l;
+  interval_bk->upper = abs((int)(fm->C[(int)c])) + occ_u - 1;
 
   return eslOK;
 }
@@ -168,8 +163,8 @@ fm_getSARangeForward( const FM_DATA *fm, FM_CFG *cfg, char *query, char *inv_alp
   FM_INTERVAL interval_bk;
 
   uint8_t c = inv_alph[(int)query[0]];
-  interval->lower  = interval_bk.lower = abs(fm->C[c]);
-  interval->upper  = interval_bk.upper = abs(fm->C[c+1])-1;
+  interval->lower  = interval_bk.lower = abs((int)(fm->C[c]));
+  interval->upper  = interval_bk.upper = abs((int)(fm->C[c+1]))-1;
 
 
   while (interval_bk.lower>=0 && interval_bk.lower <= interval_bk.upper) {
@@ -194,8 +189,8 @@ fm_updateIntervalReverse( const FM_DATA *fm, const FM_CFG *cfg, char c, FM_INTER
   count1 = fm_getOccCount (fm, cfg, interval->lower-1, c);
   count2 = fm_getOccCount (fm, cfg, interval->upper, c);
 
-  interval->lower = abs(fm->C[(int)c]) + count1;
-  interval->upper = abs(fm->C[(int)c]) + count2 - 1;
+  interval->lower = abs((int)(fm->C[(int)c])) + count1;
+  interval->upper = abs((int)(fm->C[(int)c])) + count2 - 1;
 
   return eslOK;
 }
@@ -217,8 +212,8 @@ fm_getSARangeReverse( const FM_DATA *fm, FM_CFG *cfg, char *query, char *inv_alp
   int i=0;
 
   char c = inv_alph[(int)query[0]];
-  interval->lower  = abs(fm->C[(int)c]);
-  interval->upper  = abs(fm->C[(int)c+1])-1;
+  interval->lower  = abs((int)(fm->C[(int)c]));
+  interval->upper  = abs((int)(fm->C[(int)c+1]))-1;
 
   while (interval->lower>=0 && interval->lower <= interval->upper) {
     c = query[++i];
@@ -501,12 +496,8 @@ fm_FM_read( FM_DATA *fm, FM_METADATA *meta, int getAll )
   //shortcut variables
   int64_t *C               = NULL;
 
-  int status;
   int i;
-
-  uint16_t *occCnts_b  = NULL;  //convenience variables, used to simplify macro calls
   uint32_t *occCnts_sb = NULL;
-
   int32_t compressed_bytes;
   int num_freq_cnts_b;
   int num_freq_cnts_sb;
@@ -514,22 +505,18 @@ fm_FM_read( FM_DATA *fm, FM_METADATA *meta, int getAll )
   int64_t prevC;
   int cnt;
   int chars_per_byte = 8/meta->charBits;
+  int status;
 
 
-  if(fread(&(fm->N), sizeof(uint64_t), 1, meta->fp) !=  1)
-    esl_fatal( "%s: Error reading block_length in FM index.\n", __FILE__);
-  if(fread(&(fm->term_loc), sizeof(uint32_t), 1, meta->fp) !=  1)
-    esl_fatal( "%s: Error reading terminal location in FM index.\n", __FILE__);
-  if(fread(&(fm->seq_offset), sizeof(uint32_t), 1, meta->fp) !=  1)
-    esl_fatal( "%s: Error reading seq_offset in FM index.\n", __FILE__);
-  if(fread(&(fm->ambig_offset), sizeof(uint32_t), 1, meta->fp) !=  1)
-    esl_fatal( "%s: Error reading ambig_offset in FM index.\n", __FILE__);
-  if(fread(&(fm->overlap), sizeof(uint32_t), 1, meta->fp) !=  1)
-    esl_fatal( "%s: Error reading overlap in FM index.\n", __FILE__);
-  if(fread(&(fm->seq_cnt), sizeof(uint32_t), 1, meta->fp) !=  1)
-    esl_fatal( "%s: Error reading seq_cnt in FM index.\n", __FILE__);
-  if(fread(&(fm->ambig_cnt), sizeof(uint32_t), 1, meta->fp) !=  1)
-    esl_fatal( "%s: Error reading ambig_cnt in FM index.\n", __FILE__);
+  if(fread(&(fm->N), sizeof(uint64_t), 1, meta->fp) !=  1            ||
+     fread(&(fm->term_loc), sizeof(uint32_t), 1, meta->fp) !=  1     ||
+     fread(&(fm->seq_offset), sizeof(uint32_t), 1, meta->fp) !=  1   ||
+     fread(&(fm->ambig_offset), sizeof(uint32_t), 1, meta->fp) !=  1 ||
+     fread(&(fm->overlap), sizeof(uint32_t), 1, meta->fp) !=  1      ||
+     fread(&(fm->seq_cnt), sizeof(uint32_t), 1, meta->fp) !=  1      ||
+     fread(&(fm->ambig_cnt), sizeof(uint32_t), 1, meta->fp) !=  1
+     )
+       {status=eslEFORMAT; goto ERROR;}
 
   compressed_bytes =   ((chars_per_byte-1+fm->N)/chars_per_byte);
   num_freq_cnts_b  = 1+ceil((double)fm->N/meta->freq_cnt_b);
@@ -546,32 +533,26 @@ fm_FM_read( FM_DATA *fm, FM_METADATA *meta, int getAll )
   ESL_ALLOC (fm->occCnts_sb,  num_freq_cnts_sb *  (meta->alph_size ) * sizeof(uint32_t)); // every freq_cnt positions, store an array of ints
 
 
-  if(getAll && fread(fm->T, sizeof(uint8_t), compressed_bytes, meta->fp) != compressed_bytes)
-    esl_fatal( "%s: Error reading T in FM index.\n", __FILE__);
-  if( fread(fm->BWT, sizeof(uint8_t), compressed_bytes, meta->fp)  != compressed_bytes)
-    esl_fatal( "%s: Error reading BWT in FM index.\n", __FILE__);
-  if(getAll && fread(fm->SA, sizeof(uint32_t), (size_t)num_SA_samples, meta->fp) != (size_t)num_SA_samples)
-    esl_fatal( "%s: Error reading SA in FM index.\n", __FILE__);
-
-  if(fread(fm->occCnts_b, sizeof(uint16_t)*(meta->alph_size), (size_t)num_freq_cnts_b, meta->fp) != (size_t)num_freq_cnts_b)
-    esl_fatal( "%s: Error reading occCnts_b in FM index.\n", __FILE__);
-  if(fread(fm->occCnts_sb, sizeof(uint32_t)*(meta->alph_size), (size_t)num_freq_cnts_sb, meta->fp) != (size_t)num_freq_cnts_sb)
-    esl_fatal( "%s: Error reading occCnts_sb in FM index.\n", __FILE__);
-
+  if(
+     (getAll && fread(fm->T, sizeof(uint8_t), compressed_bytes, meta->fp) != compressed_bytes) ||
+     (fread(fm->BWT, sizeof(uint8_t), compressed_bytes, meta->fp)  != compressed_bytes) ||
+     (getAll && fread(fm->SA, sizeof(uint32_t), (size_t)num_SA_samples, meta->fp) != (size_t)num_SA_samples)  ||
+     (fread(fm->occCnts_b, sizeof(uint16_t)*(meta->alph_size), (size_t)num_freq_cnts_b, meta->fp) != (size_t)num_freq_cnts_b)  ||
+     (fread(fm->occCnts_sb, sizeof(uint32_t)*(meta->alph_size), (size_t)num_freq_cnts_sb, meta->fp) != (size_t)num_freq_cnts_sb)
+    )
+    {status=eslEFORMAT; goto ERROR;}
 
   //shortcut variables
   C          = fm->C;
-  occCnts_b  = fm->occCnts_b;
   occCnts_sb = fm->occCnts_sb;
 
-
   /*compute the first position of each letter in the alphabet in a sorted list
   * (with an extra value to simplify lookup of the last position for the last letter).
   * Negative values indicate that there are zero of that character in T, can be
   * used to establish the end of the prior range*/
   C[0] = 0;
   for (i=0; i<meta->alph_size; i++) {
-    prevC = abs(C[i]);
+    prevC = abs((int)(C[i]));
 
     cnt = FM_OCC_CNT( sb, num_freq_cnts_sb-1, i);
 
@@ -589,8 +570,7 @@ fm_FM_read( FM_DATA *fm, FM_METADATA *meta, int getAll )
 
 ERROR:
   fm_FM_destroy(fm, getAll);
-  esl_fatal("Error allocating memory in %s\n", "readFM");
-  return eslFAIL;
+  return status;
 }
 
 
@@ -621,25 +601,33 @@ fm_readFMmeta( FM_METADATA *meta)
       fread(&(meta->ambig_list->count), sizeof(meta->ambig_list->count),    1, meta->fp) != 1 ||
       fread(&(meta->char_count),   sizeof(meta->char_count),   1, meta->fp) != 1
   )
-    esl_fatal( "%s: Error reading meta data for FM index.\n", __FILE__);
+  {status=eslEFORMAT; goto ERROR;}
+
+  /* sanity check - are these metadata for a real FM index?
+   * TODO: in an upcoming renovation of FM, capture FM validation & version as part of metadata header
+   */
+  if (  meta->alph_type != fm_DNA ||  /* this is the only legal value for nhmmer */
+        meta->fwd_only > 1        ||  /* must be 0 (false) or 1 (true) */
+        meta->charBits > 8        ||  /* should really be 2 ... but allowing for future growth */
+        meta->freq_SA > 10000         /* a suffix array sampling of this scale is insane */
+  )
+  {status=eslEFORMAT; return status;}
 
 
   ESL_ALLOC (meta->seq_data,  meta->seq_count   * sizeof(FM_SEQDATA));
-  if (meta->seq_data == NULL  )
-    esl_fatal("unable to allocate memory to store FM meta data\n");
 
 
   for (i=0; i<meta->seq_count; i++) {
-    if( fread(&(meta->seq_data[i].target_id),    sizeof(meta->seq_data[i].target_id),           1, meta->fp) != 1 ||
-        fread(&(meta->seq_data[i].target_start), sizeof(meta->seq_data[i].target_start),        1, meta->fp) != 1 ||
-        fread(&(meta->seq_data[i].fm_start),     sizeof(meta->seq_data[i].fm_start),        1, meta->fp) != 1 ||
+    if( fread(&(meta->seq_data[i].target_id),    sizeof(meta->seq_data[i].target_id),    1, meta->fp) != 1 ||
+        fread(&(meta->seq_data[i].target_start), sizeof(meta->seq_data[i].target_start), 1, meta->fp) != 1 ||
+        fread(&(meta->seq_data[i].fm_start),     sizeof(meta->seq_data[i].fm_start),     1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].length),       sizeof(meta->seq_data[i].length),       1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].name_length),  sizeof(meta->seq_data[i].name_length),  1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].acc_length),   sizeof(meta->seq_data[i].acc_length),   1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].source_length),sizeof(meta->seq_data[i].source_length),1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].desc_length),  sizeof(meta->seq_data[i].desc_length),  1, meta->fp) != 1
         )
-      esl_fatal( "%s: Error reading meta data for FM index.\n", __FILE__);
+        {status=eslEFORMAT; goto ERROR;}
 
     ESL_ALLOC (meta->seq_data[i].name,  (1+meta->seq_data[i].name_length)   * sizeof(char));
     ESL_ALLOC (meta->seq_data[i].acc,   (1+meta->seq_data[i].acc_length)    * sizeof(char));
@@ -650,15 +638,13 @@ fm_readFMmeta( FM_METADATA *meta)
         fread(meta->seq_data[i].name,   sizeof(char), meta->seq_data[i].name_length+1   , meta->fp) !=  meta->seq_data[i].name_length+1  ||
         fread(meta->seq_data[i].acc,    sizeof(char), meta->seq_data[i].acc_length+1    , meta->fp) !=  meta->seq_data[i].acc_length+1  ||
         fread(meta->seq_data[i].source, sizeof(char), meta->seq_data[i].source_length+1 , meta->fp) !=  meta->seq_data[i].source_length+1  ||
-        fread(meta->seq_data[i].desc,   sizeof(char), meta->seq_data[i].desc_length+1   , meta->fp) !=  meta->seq_data[i].desc_length+1 )
-      esl_fatal( "%s: Error reading meta data for FM index.\n", __FILE__);
-
+        fread(meta->seq_data[i].desc,   sizeof(char), meta->seq_data[i].desc_length+1   , meta->fp) !=  meta->seq_data[i].desc_length+1
+      )
+      {status=eslEFORMAT; goto ERROR;}
   }
 
   if (meta->ambig_list->count > meta->ambig_list->size) {
     ESL_REALLOC(meta->ambig_list->ranges,  meta->ambig_list->count  * sizeof(FM_INTERVAL));
-    if (meta->ambig_list->ranges == NULL  )
-      esl_fatal("unable to allocate memory to store FM ambiguity data\n");
     meta->ambig_list->size = meta->ambig_list->count;
   }
 
@@ -666,7 +652,7 @@ fm_readFMmeta( FM_METADATA *meta)
     if( fread(&(meta->ambig_list->ranges[i].lower),   sizeof(meta->ambig_list->ranges[i].lower),       1, meta->fp) != 1 ||
         fread(&(meta->ambig_list->ranges[i].upper),   sizeof(meta->ambig_list->ranges[i].upper),       1, meta->fp) != 1
     )
-      esl_fatal( "%s: Error reading ambiguity data for FM index.\n", __FILE__);
+    {status=eslEAMBIGUOUS; goto ERROR;}
   }
 
   return eslOK;
@@ -680,8 +666,7 @@ ERROR:
   }
   free(meta);
 
-   esl_fatal("Error allocating memory in %s\n", "readFM");
-   return eslFAIL;
+  return status;
 }
 
 
@@ -693,25 +678,12 @@ fm_configAlloc(FM_CFG **cfg)
 {
   int status;
 
-  if ( cfg == NULL)
-    esl_fatal("null pointer when allocating FM configuration\n");
-
+  if (cfg == NULL) {status=eslERANGE; goto ERROR;}
   *cfg = NULL;
 
   ESL_ALLOC(*cfg, sizeof(FM_CFG) );
-  if ((*cfg) == NULL)
-    esl_fatal("unable to allocate memory to store FM config data\n");
-
   ESL_ALLOC((*cfg)->meta, sizeof(FM_METADATA));
-  if ((*cfg)->meta == NULL)
-    esl_fatal("unable to allocate memory to store FM meta data\n");
-
   ESL_ALLOC ((*cfg)->meta->ambig_list, sizeof(FM_AMBIGLIST));
-  if ((*cfg)->meta->ambig_list == NULL)
-      esl_fatal("unable to allocate memory to store FM ambiguity data\n");
-
-
-
 
   return eslOK;
 
@@ -720,7 +692,7 @@ ERROR:
     if ((*cfg)->meta != NULL) free ((*cfg)->meta);
     free (*cfg);
   }
-  return eslEMEM;
+  return status;
 }
 
 
@@ -734,7 +706,7 @@ ERROR:
 int
 fm_configDestroy(FM_CFG *cfg ) {
   if (cfg) {
-#if   defined (p7_IMPL_SSE)
+#if defined (eslENABLE_SSE)
     if (cfg->fm_chars_mem)         free(cfg->fm_chars_mem);
     if (cfg->fm_masks_mem)         free(cfg->fm_masks_mem);
     if (cfg->fm_reverse_masks_mem) free(cfg->fm_reverse_masks_mem);
@@ -773,16 +745,5 @@ fm_metaDestroy(FM_METADATA *meta ) {
   return eslOK;
 }
 
-/************************************************************
- * HMMER - Biological sequence analysis with profile HMMs
- * Version 3.1b3; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute.
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
- * 
- * HMMER is distributed under the terms of the GNU General Public License
- * (GPLv3). See the LICENSE file for details.
- *
- * SVN $Id: fm_general.c 3784 2011-12-07 21:51:25Z wheelert $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/trunk/src/fm_general.c $
- ************************************************************/
+
 
