@@ -3,17 +3,17 @@
  * Contents:
  *    1. The <esl_msa_hmmpgmd2msa> function
  *    2. Test driver
- *    3. Copyright and license information
- *
  */
-
+#include "p7_config.h"
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <hmmer.h>
-#include <hmmpgmd.h>
-#include <esl_sqio.h>
+
+#include "hmmer.h"
+#include "hmmpgmd.h"
+
+#include "esl_sqio.h"
 
 
 /******************************************************************************
@@ -72,32 +72,29 @@
  *
  */
 int
-hmmpgmd2msa(void *data, P7_HMM *hmm, ESL_SQ *qsq, int *incl, int incl_size, int *excl, int excl_size, ESL_MSA **ret_msa) {
+hmmpgmd2msa(void *data, P7_HMM *hmm, ESL_SQ *qsq, int *incl, int incl_size, int *excl, int excl_size, int excl_all, ESL_MSA **ret_msa) {
   int i, j;
   int c;
   int status;
-  int set_included;
-
+  uint32_t n = 0;
   /* trace of the query sequence with N residues onto model with N match states */
   P7_TRACE          *qtr         = NULL;
   int                extra_sqcnt = 0;
 
   /* vars used to read from the binary data */
   HMMD_SEARCH_STATS *stats   = NULL;              /* pointer to a single stats object, at the beginning of data */
-  P7_HIT            *hits    = NULL;              /* an array of hits, at the appropriate offset in data */
 
   /* vars used in msa construction */
   P7_TOPHITS         th;
-  P7_ALIDISPLAY     *ad, *ad2;
   ESL_MSA           *msa   = NULL;
-  P7_DOMAIN         *dom   = NULL;
 
   char              *p     = (char*)data;        /*pointer used to walk along data, must be char* to allow pointer arithmetic */
 
   th.N = 0;
   th.unsrt = NULL;
   th.hit   = NULL;
-
+  ESL_ALLOC(stats, sizeof(HMMD_SEARCH_STATS));
+  stats->hit_offsets = NULL; // we don't use hit_offsets for this test
   /* optionally build a faux trace for the query sequence: relative to core model (B->M_1..M_L->E) */
   if (qsq != NULL) {
     if (qsq->n != hmm->M) {
@@ -116,7 +113,10 @@ hmmpgmd2msa(void *data, P7_HMM *hmm, ESL_SQ *qsq, int *incl, int incl_size, int
   }
 
   /* get search stats + hit info */
-  stats = (HMMD_SEARCH_STATS*)p;
+  if(p7_hmmd_search_stats_Deserialize((uint8_t *) p, &n, stats) != eslOK){
+      status = eslFAIL;
+    goto ERROR;
+  }
 
   /* sanity check */
   if (   ( stats->Z_setby != p7_ZSETBY_NTARGETS    && stats->Z_setby != p7_ZSETBY_OPTION    && stats->Z_setby != p7_ZSETBY_FILEINFO )
@@ -129,13 +129,22 @@ hmmpgmd2msa(void *data, P7_HMM *hmm, ESL_SQ *qsq, int *incl, int incl_size, int
   }
 
   /* ok, it looks legitimate */
-  p    += sizeof(HMMD_SEARCH_STATS);
-  hits  = (P7_HIT*)p;
-  p    += sizeof(P7_HIT) * stats->nhits;
-
   /* create a tophits object, to be passed to p7_tophits_Alignment() */
   ESL_ALLOC( th.unsrt, sizeof(P7_HIT) * stats->nhits);
-  memcpy( th.unsrt, hits, sizeof(P7_HIT) * stats->nhits);
+  // deserialize all the hits
+  for (i = 0; i < stats->nhits; ++i) {
+    // set all internal pointers of the hit to NULL before deserializing into it
+    th.unsrt[i].name = NULL;
+    th.unsrt[i].acc = NULL;
+    th.unsrt[i].desc = NULL;
+    th.unsrt[i].dcl = NULL;
+
+    if(p7_hit_Deserialize((uint8_t *) p, &n, &(th.unsrt[i])) != eslOK){
+      printf("Unable to deserialize hit %d\n", i);
+      exit(0);
+    }  
+  }
+
   ESL_ALLOC( th.hit, sizeof(P7_HIT*) * stats->nhits);
   for (i=0; i<stats->nhits; i++) {
     th.hit[i] = &(th.unsrt[i]);
@@ -154,10 +163,19 @@ hmmpgmd2msa(void *data, P7_HMM *hmm, ESL_SQ *qsq, int *incl, int incl_size, int
   th.is_sorted_by_sortkey = 0;
   th.is_sorted_by_seqidx  = 0;
 
+  /* jackhmmer (hmmer web) - allow all hits to be unchecked */
+  if(excl_all){
+    for (i = 0; i < th.N; i++) {
+      /* Go through the hits and set all to be excluded */
+      if(th.hit[i]->flags & p7_IS_INCLUDED){
+        th.hit[i]->flags = p7_IS_DROPPED;
+        th.hit[i]->nincluded = 0;
+      }
+    }
+  }
+
   for (i = 0; i < th.N; i++) {
-    ESL_ALLOC( th.hit[i]->dcl, sizeof(P7_DOMAIN) *  th.hit[i]->ndom);
     /* Go through the hits and set to be excluded or included as necessary */
-    set_included = 0;
     if(th.hit[i]->flags & p7_IS_INCLUDED){
       if(excl_size > 0){
         for( c = 0; c < excl_size; c++){
@@ -173,65 +191,19 @@ hmmpgmd2msa(void *data, P7_HMM *hmm, ESL_SQ *qsq, int *incl, int incl_size, int
     	for( c = 0; c < incl_size; c++){
           if(incl[c] == (long)th.hit[i]->name ){
             th.hit[i]->flags = p7_IS_INCLUDED;
-            set_included = 1;
           }
         }
       }
     }
-    /* first grab all the P7_DOMAINs for the hit */
-    for (j=0; j < th.hit[i]->ndom; j++) {
-      dom = th.hit[i]->dcl + j;
-      memcpy(dom , (P7_DOMAIN*)p, sizeof(P7_DOMAIN));
-      /* Possibly set domains to be include if being
-       * externally set via incl list*/
-      if(set_included) th.hit[i]->dcl[j].is_included = 1;
-      p += sizeof(P7_DOMAIN);
-    }
-    /* then grab the P7_ALIDISPLAYs for the hit */
-    for (j=0; j < th.hit[i]->ndom; j++) {
-      ad = (P7_ALIDISPLAY*)p;
-      ESL_ALLOC(th.hit[i]->dcl[j].ad, sizeof(P7_ALIDISPLAY));
-      ad2 = th.hit[i]->dcl[j].ad;
-
-      ad2->memsize = ad->memsize;
-      ad2->rfline = ad->rfline;
-      ad2->mmline = ad->mmline;
-      ad2->csline = ad->csline ;
-      ad2->model  = ad->model ;
-      ad2->mline  = ad->mline ;
-      ad2->aseq   = ad->aseq ;
-      ad2->ppline = ad->ppline;
-      ad2->N      = ad->N;
-
-      ad2->hmmname = ad->hmmname;
-      ad2->hmmacc  = ad->hmmacc ;
-      ad2->hmmdesc = ad->hmmdesc;
-      ad2->hmmfrom = ad->hmmfrom;
-      ad2->hmmto   = ad->hmmto;
-      ad2->M       = ad->M;
-
-      ad2->sqname  = ad->sqname;
-      ad2->sqacc   = ad->sqacc ;
-      ad2->sqdesc  = ad->sqdesc;
-      ad2->sqfrom  = ad->sqfrom;
-      ad2->sqto    = ad->sqto;
-      ad2->L       = ad->L;
-
-      p += sizeof(P7_ALIDISPLAY);
-
-      ESL_ALLOC(ad2->mem, ad2->memsize);
-      
-      memcpy(ad2->mem, p, ad->memsize);
-      
-      p += ad2->memsize;
-      p7_alidisplay_Deserialize(ad2);
-    }
   }
 
 
   /* use the tophits and trace info above to produce an alignment */
   if ( (status = p7_tophits_Alignment(&th, hmm->abc, &qsq, &qtr, extra_sqcnt, p7_ALL_CONSENSUS_COLS, &msa)) != eslOK) goto ERROR;
-
+  esl_msa_SetName     (msa, hmm->name, -1);
+  esl_msa_SetAccession(msa, hmm->acc,  -1);
+  esl_msa_SetDesc     (msa, hmm->desc, -1);
+  esl_msa_FormatAuthor(msa, "hmmpgmd (HMMER %s)", HMMER_VERSION);
 
   /* free memory */
   if (qtr != NULL) free(qtr);
@@ -243,7 +215,7 @@ hmmpgmd2msa(void *data, P7_HMM *hmm, ESL_SQ *qsq, int *incl, int incl_size, int
   }
   if (th.unsrt != NULL) free (th.unsrt);
   if (th.hit != NULL) free (th.hit);
-
+  free(stats);
   *ret_msa = msa;
   return eslOK;
 
@@ -259,7 +231,7 @@ ERROR:
   }
   if (th.unsrt != NULL) free (th.unsrt);
   if (th.hit != NULL) free (th.hit);
-
+  if(stats != NULL) free(stats);
   return status;
 }
 
@@ -268,7 +240,7 @@ ERROR:
  *****************************************************************************/
 
 
-
+// This function is believed to be defunct.  Name has been changed by adding _old so that EBI can verify
 /* Function:  hmmpgmd2stats()
  * Synopsis:  Use a HMMPGMD-derived data stream to extract some simple
  *            statistics regarding its alignment.
@@ -292,7 +264,7 @@ ERROR:
  *                      caller is responsible for freeing it later   
  *
  */
-int hmmpgmd2stats(void *data, P7_HMM *hmm, float** statsOut) 
+int hmmpgmd2stats_old(void *data, P7_HMM *hmm, float** statsOut) 
 {
   int i, j, k;
   int status;
@@ -427,7 +399,7 @@ int hmmpgmd2stats(void *data, P7_HMM *hmm, float** statsOut)
       
       p += ad2->memsize;
       
-      p7_alidisplay_Deserialize(ad2);
+      p7_alidisplay_Deserialize_old(ad2);
       
       if(th.hit[i]->flags & p7_IS_INCLUDED && th.hit[i]->dcl[j].is_included)
       {
@@ -547,121 +519,154 @@ ERROR:
  *****************************************************************/
 
 //#define hmmpgmd2msa_TESTDRIVE
-#ifdef hmmpgmd2msa_TESTDRIVE
+#ifdef p7HMMPGMD2MSA_TESTDRIVE
+#define NUM_TEST_SEQUENCES 20
+
+void hmmpgmd2msa_utest(int ntrials, char *hmmfile){
+  char msg[] = "hmmpgmd2msa_utest failed";
+  ESL_RANDOMNESS *rng = esl_randomness_Create(0);
+  P7_HMM *hmm;
+  ESL_ALPHABET *abc = esl_alphabet_Create(eslAMINO);
+  P7_OPROFILE *oprofile;
+  P7_TOPHITS *hitlist; 
+  P7_HMMFILE   *hfp     = NULL;
+  P7_BG *bg  = p7_bg_Create(abc);
+  ESL_MSA *msa, *deser_msa;
+  ESL_SQ * sequences[NUM_TEST_SEQUENCES];
+  uint8_t **buf, *buf_data;
+  HMMD_SEARCH_STATS stats;
+  uint32_t n = 0;
+  uint32_t nalloc = 0;
+
+  buf_data = NULL;
+  buf = &(buf_data);
+
+  // Grab the HMM from the input file
+  if (p7_hmmfile_OpenE(hmmfile, NULL, &hfp, NULL) != eslOK) p7_Fail("Failed to open HMM file %s", hmmfile);
+  if (p7_hmmfile_Read(hfp, &abc, &hmm)            != eslOK) p7_Fail("Failed to read HMM");
+  p7_hmmfile_Close(hfp);
+
+  P7_PROFILE *profile = p7_profile_Create(hmm->M, abc);
+  oprofile = p7_oprofile_Create(hmm->M, abc);
+
+  if(p7_ProfileConfig(hmm, bg, profile, 400, p7_LOCAL) != eslOK){
+    p7_Fail(msg);
+  }
+  if(p7_oprofile_Convert(profile, oprofile) != eslOK){
+    p7_Fail(msg);
+  }
+
+  // build a pipeline to search them with
+  P7_PIPELINE *pli=p7_pipeline_Create(NULL, hmm->M, 400, FALSE, p7_SEARCH_SEQS);
+  p7_pli_NewModel(pli, oprofile, bg);
+
+  for(int j = 0; j < ntrials; j++){
+    
+    // sample sequences from the HMM
+    for(int i = 0; i < NUM_TEST_SEQUENCES; i++){
+      sequences[i] = esl_sq_CreateDigital(abc);
+      p7_ProfileEmit(rng, hmm, profile, bg, sequences[i], NULL);
+    }
+
+    hitlist = p7_tophits_Create();
+
+    // Search all of the sampled sequences against the HMM to generate a set of hits
+    for(int i = 0; i < NUM_TEST_SEQUENCES; i++){
+      p7_pli_NewSeq(pli, sequences[i]);
+      p7_bg_SetLength(bg, sequences[i]->n);
+      p7_oprofile_ReconfigLength(oprofile, sequences[i]->n);
+  
+      p7_Pipeline(pli, oprofile, bg, sequences[i], NULL, hitlist);
+      p7_pipeline_Reuse(pli);
+    }
+
+
+    // Replicate the process hmmpgmd2msa uses to create an MSA so we have something to compare to
+    p7_tophits_SortBySortkey(hitlist);
+    p7_tophits_Threshold(hitlist, pli);
+    if (p7_tophits_Alignment(hitlist, abc, NULL, NULL, 0, p7_ALL_CONSENSUS_COLS, &msa) != eslOK){
+      p7_Fail(msg);
+    }
+
+    esl_msa_SetName     (msa, hmm->name, -1);
+    esl_msa_SetAccession(msa, hmm->acc,  -1);
+    esl_msa_SetDesc     (msa, hmm->desc, -1);
+    esl_msa_FormatAuthor(msa, "hmmpgmd (HMMER %s)", HMMER_VERSION);
+
+
+    // ok, we have an MSA generated directly from the hits.  Now, fake up a blob of serialized data as hmmpgmd would have sent
+    stats.elapsed = esl_random(rng);
+    stats.user = esl_random(rng);
+    stats.sys = esl_random(rng);
+    stats.Z = pli->Z;
+    stats.domZ = pli->domZ;
+    stats.Z_setby = pli->Z_setby;
+    stats.domZ_setby = pli->domZ_setby;
+    stats.nmodels = pli->nmodels;
+    stats.nseqs = pli->nseqs;
+    stats.n_past_msv = pli->n_past_msv;
+    stats.n_past_bias = pli->n_past_bias;
+    stats.n_past_vit = pli->n_past_vit;
+    stats.n_past_fwd = pli->n_past_fwd;
+    stats.nhits = hitlist->N;
+    stats.nreported = hitlist->nreported;
+    stats.nincluded = hitlist->nreported;
+    stats.hit_offsets = NULL; // don't need this for hmmpgmd2msa
+
+    if(p7_hmmd_search_stats_Serialize(&stats, buf, &n, &nalloc) !=eslOK){
+      p7_Fail(msg);
+    }
+
+    for(int k = 0; k < hitlist->N; k++){
+      if(p7_hit_Serialize(hitlist->hit[k], buf, &n, &nalloc) !=eslOK){
+        p7_Fail(msg);
+      }
+    }
+
+    if(hmmpgmd2msa(buf_data, hmm, NULL, NULL, 0, NULL, 0, 0, &deser_msa) != eslOK){
+      p7_Fail(msg);
+    }
+
+    // compare the MSAs
+    if(esl_msa_Compare (msa, deser_msa) != eslOK){
+      p7_Fail(msg);
+    }
+
+    // clean up after ourselves 
+    p7_tophits_Destroy(hitlist);
+    for(int k = 0; k < NUM_TEST_SEQUENCES; k++){
+      esl_sq_Destroy(sequences[k]);
+      sequences[k] = NULL;
+    }
+    esl_msa_Destroy(msa);
+    esl_msa_Destroy(deser_msa);
+    n = 0; // reset this to the start of the buffer for next time
+  }
+
+
+
+  // final cleanup
+  p7_pipeline_Destroy(pli);
+  p7_profile_Destroy(profile);
+  p7_oprofile_Destroy(oprofile);
+  p7_hmm_Destroy(hmm);
+  p7_bg_Destroy(bg);
+  esl_alphabet_Destroy(abc);
+  esl_randomness_Destroy(rng);
+  free(buf_data);
+}
 
-//gcc -o alistat_test -msse2 -std=gnu99 -g -O2 -I. -L. -I../easel -L../easel -D hmmpgmd2msa_TESTDRIVE hmmpgmd2msa.c -lhmmer -leasel -lm
 
-/* Test driver. As written, requires files that won't be released with
- * the distribution. So it should be replaced with a tighter test.
- */
 int
 main(int argc, char **argv) {
-  ESL_MSA           *msa   = NULL;
-  ESL_SQ            *qsq   = NULL;
-  ESL_SQFILE        *qfp   = NULL;              /* open qfile                                      */
-  P7_HMMFILE        *hfp   = NULL;              /* open input HMM file                             */
-  P7_HMM            *hmm   = NULL;              /* one HMM query                                   */
-  ESL_ALPHABET      *abc   = NULL;              /* digital alphabet                                */
-  FILE              *fp    = NULL;              /* open file containing the HMMPGMD data */
-  void              *data  = NULL;              /* pointer to the full data stream built as if from hmmdmstr */
-  long size = 0;
-
-  char   errbuf[eslERRBUFSIZE];
-  int status;
-  
-  float* statsOut;
-  int x;
-  //char *badstring = "asdlfuhasdfuhasdfhasdfhaslidhflaishdfliasuhdfliasuhdfliasudfh";
-
-  char *hmm_file = "esl_align.hmm";
-  char *fa_file  = "esl_align.fa";
-  char *dat_file = "esl_align.big.bin";
-
-  if (argc > 1 ) {
-    ESL_ALLOC(hmm_file, sizeof(char) * (strlen(argv[1])+1) );
-    strcpy(hmm_file, argv[1]);
+  if(argc != 2){
+    printf("Usage: hmmpgmd2msa_utest <hmmfile>\n");
+    return eslFAIL;
   }
-  if (argc > 2 ) {
-    ESL_ALLOC(dat_file, sizeof(char) * (strlen(argv[2])+1) );
-    strcpy(dat_file, argv[2]);
-  }
-  if (argc > 3 ) {
-    ESL_ALLOC(fa_file, sizeof(char) * (strlen(argv[3])+1) );
-    strcpy(fa_file, argv[3]);
-  }
-
-  printf("hmmpgmd2msa:\nhmm: %s\nfa:  %s\ndat: %s\n", hmm_file, fa_file, dat_file);
-
-
-  /* read the hmm */
-  if ( (status = p7_hmmfile_OpenE(hmm_file, NULL, &hfp, errbuf)) != 0 ) goto ERROR;
-  if ( (status = p7_hmmfile_Read(hfp, &abc, &hmm)) != 0 ) goto ERROR;
-
-  /* read the query sequence */
-  //if ( (status = esl_sqfile_OpenDigital(abc, fa_file, eslSQFILE_UNKNOWN, NULL, &qfp)) != 0) goto ERROR;
-  //qsq = esl_sq_CreateDigital(abc);
-  //if ( (status = esl_sqio_Read(qfp, qsq)) != eslOK)  goto ERROR;
-
-  //printf("sequence length %d\n", qsq->n);
-
-  /* get stats for the hmmd data */
-
-  if ( (fp = fopen(dat_file, "rb")) == NULL ) goto ERROR;
-
-  fseek (fp , 0 , SEEK_END);
-  size = ftell (fp);
-  rewind (fp);
-  ESL_ALLOC(data, size);
-  fread(data, size, 1, fp);
-
-  status = hmmpgmd2stats(data, hmm, &statsOut);
-  //status = hmmpgmd2msa(data, hmm, qsq, NULL,0, NULL, 0, &msa);
-
-  for(x = 0; x < hmm->M*3; x++)
-  {
-    if(statsOut[x] > 1)
-    {
-      printf("problem x: %d %f\n", x, statsOut[x]);
-    }
-  }
-
-  for(x = 0; x < hmm->M; x++)
-  {
-    printf("%d", ((int)(10*statsOut[x])>=10)?9:(int)(10*statsOut[x]));
-  }
-  printf("\n");
-  for(x = hmm->M; x < hmm->M*2; x++)
-  {
-    printf("%d", ((int)(10*statsOut[x])>=10)?9:(int)(10*statsOut[x]));
-  }
-  printf("\n");
-  for(x = hmm->M*2; x < hmm->M*3; x++)
-  {
-    printf("%d", ((int)(10*statsOut[x])>=10)?9:(int)(10*statsOut[x]));
-  }
-  printf("\n");
-
-  if (status != eslOK) goto ERROR;
-
-  //esl_msafile_Write(stdout, msa, eslMSAFILE_STOCKHOLM);
-
-  exit(0);
-
-ERROR:
-  printf ("fail!\n");
-  exit(1);
-
+  hmmpgmd2msa_utest(10, argv[1]);
+  return eslOK;  // hmmpgmd2msa_utest fails if anything goes wrong, so getting here = pass
 }
 #endif
 
-/************************************************************
- * HMMER - Biological sequence analysis with profile HMMs
- * Version 3.1b3; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute.
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
- * 
- * HMMER is distributed under the terms of the GNU General Public License
- * (GPLv3). See the LICENSE file for details.
- ************************************************************/
 
 
