@@ -5,11 +5,8 @@
  *   2. Benchmark driver.
  *   3. Unit tests.
  *   4. Test driver.
- *   5. Example.
- *   6. Copyright and license information.
  *
  * SRE, Mon Aug 18 08:31:11 2008 [Janelia]
- * SVN $Id$
  */
 #include "p7_config.h"
 
@@ -145,7 +142,7 @@ p7_Null2_ByTrace(const P7_OPROFILE *om, const P7_TRACE *tr, int zstart, int zend
   float *xmx = wrk->xmx;	/* enables use of XMXo macro */
   float  norm;
   float  xfactor;
-  int    q, r, s;
+  int    q, r;
   int    x;
   int    z;
 
@@ -172,13 +169,11 @@ p7_Null2_ByTrace(const P7_OPROFILE *om, const P7_TRACE *tr, int zstart, int zend
       Ld++;
       if (tr->k[z] > 0)	/* must be an M or I */
 	{ /* surely there's an easier way? but our workspace is striped, interleaved quads... */
-	  s = ( (tr->st[z] == p7T_M) ?  p7X_M : p7X_I);
 	  q = p7X_NSCELLS * ( (tr->k[z] - 1) % Q) + p7X_M;
 	  r = (tr->k[z] - 1) / Q;
 	  u.v            = wrk->dpf[0][q];
 	  u.p[r]        += 1.0;	/* all this to increment a count by one! */
 	  wrk->dpf[0][q] = u.v;
-
 	}
       else /* emitted an x_i with no k; must be an N,C,J */
 	{
@@ -505,23 +500,3 @@ main(int argc, char **argv)
 
 
 
-/*****************************************************************
- * 5. Example
- *****************************************************************/
-#ifdef p7NULL2_EXAMPLE
-
-#endif /*p7NULL2_EXAMPLE*/
-/*------------------------ example ------------------------------*/
-
-
-
-/*****************************************************************
- * HMMER - Biological sequence analysis with profile HMMs
- * Version 3.1b3; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute.
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
- * 
- * HMMER is distributed under the terms of the GNU General Public License
- * (GPLv3). See the LICENSE file for details.
- *****************************************************************/
-
