@@ -31,7 +31,6 @@
  * all the necessary working memory and heuristic thresholds.
  *   
  * SRE, Thu Jan 24 09:28:01 2008 [Janelia]
- * SVN $Id: p7_domaindef.c 4016 2012-05-01 03:56:41Z wheelert $
  */
 #include "p7_config.h"
 
@@ -47,8 +46,8 @@
 
 static int is_multidomain_region  (P7_DOMAINDEF *ddef, int i, int j);
 static int region_trace_ensemble  (P7_DOMAINDEF *ddef, const P7_OPROFILE *om, const ESL_DSQ *dsq, int ireg, int jreg, const P7_OMX *fwd, P7_OMX *wrk, int *ret_nc);
-static int rescore_isolated_domain(P7_DOMAINDEF *ddef, P7_DOMAINDEF *ddef_app, const P7_OPROFILE *om, const ESL_SQ *sq, P7_OMX *ox1, P7_OMX *ox2,
-				   int i, int j, int null2_is_done, P7_BG *bg, int long_target);
+static int rescore_isolated_domain(P7_DOMAINDEF *ddef, P7_OPROFILE *om, const ESL_SQ *sq, const ESL_SQ *ntsq, P7_OMX *ox1, P7_OMX *ox2,
+				   int i, int j, int null2_is_done, P7_BG *bg, int long_target, P7_BG *bg_tmp, float *scores_arr, float *fwd_emissions_arr);
 
 
 /*****************************************************************
@@ -197,10 +196,10 @@ p7_domaindef_Reuse(P7_DOMAINDEF *ddef)
   else
     {
       for (d = 0; d < ddef->ndom; d++) {
-	p7_alidisplay_Destroy(ddef->dcl[d].ad);
-	ddef->dcl[d].ad = NULL;
+	p7_alidisplay_Destroy(ddef->dcl[d].ad); ddef->dcl[d].ad             = NULL;
+	free(ddef->dcl[d].scores_per_pos);      ddef->dcl[d].scores_per_pos = NULL;
       }
-
+      
     }
   ddef->ndom = 0;
   ddef->L    = 0;
@@ -284,7 +283,6 @@ void
 p7_domaindef_Destroy(P7_DOMAINDEF *ddef)
 {
   int d;
-
   if (ddef == NULL) return;
 
   if (ddef->mocc != NULL) free(ddef->mocc);
@@ -293,8 +291,10 @@ p7_domaindef_Destroy(P7_DOMAINDEF *ddef)
   if (ddef->n2sc != NULL) free(ddef->n2sc);
 
   if (ddef->dcl  != NULL) {
-    for (d = 0; d < ddef->ndom; d++)
+    for (d = 0; d < ddef->ndom; d++) {
+      if (ddef->dcl[d].scores_per_pos) free(ddef->dcl[d].scores_per_pos);
       p7_alidisplay_Destroy(ddef->dcl[d].ad);
+    }
     free(ddef->dcl);
   }
 
@@ -328,7 +328,7 @@ p7_domaindef_Destroy(P7_DOMAINDEF *ddef)
  * Returns:   <eslOK> on success.
  */
 int
-p7_domaindef_ByViterbi(P7_PROFILE *gm, const ESL_SQ *sq, P7_GMX *gx1, P7_GMX *gx2, P7_DOMAINDEF *ddef) 
+p7_domaindef_ByViterbi(P7_PROFILE *gm, const ESL_SQ *sq, const ESL_SQ *ntsq, P7_GMX *gx1, P7_GMX *gx2, P7_DOMAINDEF *ddef) 
 {
   int            d;
   int            saveL     = gm->L;    /* need to be able to restore original <gm> config */
@@ -341,7 +341,7 @@ p7_domaindef_ByViterbi(P7_PROFILE *gm, const ESL_SQ *sq, P7_GMX *gx1, P7_GMX *gx
   p7_ReconfigUnihit(gm, 0);	  /* process each domain in unihit L=0 mode */
 
   for (d = 0; d < ddef->gtr->ndom; d++)
-    rescore_isolated_domain(ddef, gm, sq, gx1, gx2, ddef->gtr->sqfrom[d], ddef->gtr->sqto[d], FALSE, NULL, FALSE);
+    rescore_isolated_domain(ddef, gm, sq, ntsq, gx1, gx2, ddef->gtr->sqfrom[d], ddef->gtr->sqto[d], FALSE, NULL, FALSE, NULL, NULL, NULL);
 
   /* Restore original model configuration, including length */
   if (p7_IsMulti(save_mode))  p7_ReconfigMultihit(gm, saveL); 
@@ -363,12 +363,9 @@ p7_domaindef_ByViterbi(P7_PROFILE *gm, const ESL_SQ *sq, P7_GMX *gx1, P7_GMX *gx
  *            calculations) and obtain an optimal accuracy alignment,
  *            using <fwd> and <bck> matrices as workspace for the
  *            necessary full-matrix DP calculations. Caller provides a
- *            new or reused <ddef> object to hold these results, and
- *            optionally provides a new or reused <ddef_app> object to
- *            hold details required by nhmmer to produce the APP
- *            (posterior probability of being aligned) line in
- *            alignment printout. A <bg> is provided for (possible) use
- *            in null3 score correction (used in nhmmer), and a boolean
+ *            new or reused <ddef> object to hold these results.
+ *            A <bg> is provided for (possible) use in biased-composition
+ *            score correction (used in nhmmer), and a boolean
  *            <long_target> argument is provided to allow nhmmer-
  *            specific modifications to the behavior of this function
  *            (TRUE -> from nhmmer).
@@ -376,7 +373,6 @@ p7_domaindef_ByViterbi(P7_PROFILE *gm, const ESL_SQ *sq, P7_GMX *gx1, P7_GMX *gx
  *            Upon return, <ddef> contains the definitions of all the
  *            domains: their bounds, their null-corrected Forward
  *            scores, and their optimal posterior accuracy alignments.
- *            If <ddef_app> is not null, it is now prepared for reuse.
  *            
  * Returns:   <eslOK> on success.           
  *            
@@ -385,9 +381,10 @@ p7_domaindef_ByViterbi(P7_PROFILE *gm, const ESL_SQ *sq, P7_GMX *gx1, P7_GMX *gx
  *            models.
  */
 int
-p7_domaindef_ByPosteriorHeuristics(const ESL_SQ *sq, P7_OPROFILE *om, 
+p7_domaindef_ByPosteriorHeuristics(const ESL_SQ *sq, const ESL_SQ *ntsq, P7_OPROFILE *om,
 				   P7_OMX *oxf, P7_OMX *oxb, P7_OMX *fwd, P7_OMX *bck, 
-				   P7_DOMAINDEF *ddef, P7_DOMAINDEF *ddef_app, P7_BG *bg, int long_target)
+				   P7_DOMAINDEF *ddef, P7_BG *bg, int long_target,
+				   P7_BG *bg_tmp, float *scores_arr, float *fwd_emissions_arr)
 {
   int i, j;
   int triggered;
@@ -408,8 +405,10 @@ p7_domaindef_ByPosteriorHeuristics(const ESL_SQ *sq, P7_OPROFILE *om,
   p7_oprofile_ReconfigUnihit(om, saveL);	   /* process each domain in unihit mode, regardless of om->mode     */
   i     = -1;
   triggered = FALSE;
+
   for (j = 1; j <= sq->n; j++)
   {
+
     if (! triggered)
     {			/* xref J2/101 for what the logic below is: */
       if       (ddef->mocc[j] - (ddef->btot[j] - ddef->btot[j-1]) <  ddef->rt2) i = j;
@@ -469,7 +468,7 @@ p7_domaindef_ByPosteriorHeuristics(const ESL_SQ *sq, P7_OPROFILE *om,
 
                   /*the !long_target argument will cause the function to recompute null2
                    * scores if this is part of a long_target (nhmmer) pipeline */
-                  if (rescore_isolated_domain(ddef, ddef_app, om, sq, fwd, bck, i2, j2, TRUE, bg, long_target) == eslOK)
+                  if (rescore_isolated_domain(ddef, om, sq, ntsq, fwd, bck, i2, j2, TRUE, bg, long_target, bg_tmp, scores_arr, fwd_emissions_arr) == eslOK)
                        last_j2 = j2;
             }
             p7_spensemble_Reuse(ddef->sp);
@@ -479,13 +478,14 @@ p7_domaindef_ByPosteriorHeuristics(const ESL_SQ *sq, P7_OPROFILE *om,
         {
             /* The region looks simple, single domain; convert the region to an envelope. */
             ddef->nenvelopes++;
-            rescore_isolated_domain(ddef, ddef_app, om, sq, fwd, bck, i, j, FALSE, bg, long_target);
+            rescore_isolated_domain(ddef, om, sq, ntsq, fwd, bck, i, j, FALSE, bg, long_target, bg_tmp, scores_arr, fwd_emissions_arr);
         }
         i     = -1;
         triggered = FALSE;
     }
   }
 
+
   /* Restore model to uni/multihit mode, and to its original length model */
   if (p7_IsMulti(save_mode)) p7_oprofile_ReconfigMultihit(om, saveL); 
   else                       p7_oprofile_ReconfigUnihit  (om, saveL); 
@@ -677,6 +677,79 @@ region_trace_ensemble(P7_DOMAINDEF *ddef, const P7_OPROFILE *om, const ESL_DSQ *
   return eslOK;
 }
 
+
+
+
+/* Function:  reparameterize_model()
+ *
+ * Synopsis:  Establish new background priors based on a sequence window,
+ *            and change match state emission log-odds scores accordingly.
+ *
+ * Purpose:    Compute new background priors based on a sequence window,
+ *             and set match search model's match state emission log-odds
+ *             scores accordingly. Used narrowly within the post-fwd
+ *             portion of the longtarget pipeline
+ *
+ *             If sq != NULL: Given a sequence <sq> and <start> and length
+ *             <L>, compute the residue frequency, and modify <bg> in place
+ *             to store a mixture of that frequency with the default (passed
+ *             in <bg>). Then update the match emission scores in place in
+ *             <om> to account for new <bg> values. Prior bg values are
+ *             stored for return in <bgf_arr>. This is called by
+ *             rescore_isolated_domain(), which is required to call it again
+ *             once complete to return <bg> and <om> to original state.
+ *
+ *
+ *             If sq == NULL: return <bg> and <om> to original state.
+ *
+ *             Only used in the longtarget (nhmmer) case. In-place
+ *             modification is done to avoid rampant memory allocation.
+ *             Doing this requires that (a) each thread has its own
+ *             independent copy of <bg> and <om>, and (b) those are
+ *             returned to their original state before being used
+ *             outside the function using the modified structures.
+ *
+ *             The pre-allocated array <sc_tmp> must be passed, for use
+ *             in p7_oprofile_UpdateFwdEmissionScores().
+ *
+ */
+static int
+reparameterize_model (P7_BG *bg, P7_OPROFILE *om, const ESL_SQ *sq, int start, int L, float *fwd_emissions, float *bgf_arr, float *sc_arr) {
+  int     K   = om->abc->K;
+  int i;
+  float tmp;
+  int status;
+
+  /* Fraction of new bg frequencies that comes from a prior determined by the sequence block.
+   * This is 25% for long sequences, more for shorter sequences (e.g. 50% for sequences of length 50)
+   */
+  float   bg_smooth = 1.; // will be modified immediately below, if it's used
+
+  if (sq != NULL) {
+    /* compute new bg->f, capturing original values into a preallocated array */
+    bg_smooth = 25.0 / (ESL_MIN(100,ESL_MAX(50,sq->n)));
+
+    esl_vec_FSet (bgf_arr, om->abc->K, 0);
+    status = esl_sq_CountResidues(sq, start, L, bgf_arr);
+    if (status != eslOK) p7_Fail("Invalid sequence range in reparameterize_model()\n");
+    esl_vec_FNorm(bgf_arr, om->abc->K);
+
+    for (i=0; i<K; i++) {
+       tmp = bg->f[i];
+       bg->f[i] = (bg_smooth*bg->f[i]) + ( (1.0-bg_smooth) * bgf_arr[i])  ;
+       bgf_arr[i] = tmp;
+    }
+  } else {
+    /* revert bg->f to the passed in orig_bgf   */
+    esl_vec_FCopy(bgf_arr, K, bg->f);
+  }
+
+  p7_oprofile_UpdateFwdEmissionScores(om, bg, fwd_emissions, sc_arr);
+
+  return eslOK;
+}
+
+
 /* rescore_isolated_domain()
  * SRE, Fri Feb  8 09:18:33 2008 [Janelia]
  *
@@ -701,15 +774,14 @@ region_trace_ensemble(P7_DOMAINDEF *ddef, const P7_OPROFILE *om, const ESL_DSQ *
  * sufficient for the complete sequence lying around, and can just use
  * those.) The caller also provides a <P7_DOMAINDEF> object (ddef)
  * which is (efficiently, we trust) managing any necessary temporary
- * working space and heuristic thresholds. The caller also optionally
- * provides a second <P7_DOMAINDEF> object (ddef_app) which if
- * provided is used to hold details required by nhmmer to produce the
- * APP (posterior probability of being aligned) line in alignment
- * printout.
+ * working space and heuristic thresholds.
  *
- * If <long_target> is TRUE, null3 biased-composition score correction
- * is used, and <bg> is required. Otherwise null2 is used, and <bg>
- * may be NULL.
+ * If <long_target> is TRUE, the calling function  optionally
+ * passes in three allocated arrays (bg_tmp, scores_arr,
+ * fwd_emissions_arr) used for temporary storage in
+ * reparameterize_model(). If scores_arr is NULL, reparameterization
+ * is not done, and the domcorrection, used to determine null2, is not
+ * computed).
  * 
  * Returns <eslOK> if a domain was successfully identified, scored,
  * and aligned in the envelope; if so, the per-domain information is
@@ -720,8 +792,6 @@ region_trace_ensemble(P7_DOMAINDEF *ddef, const P7_OPROFILE *om, const ESL_DSQ *
  * <ddef>: <ddef->tr> has been used, and possibly reallocated, for
  *         the OA trace of the domain. Before exit, we called
  *         <Reuse()> on it.
- *
- * <ddef_app>: if not null, it is now prepared for reuse.
  * 
  * <ox1> : happens to be holding OA score matrix for the domain
  *         upon return, but that's not part of the spec; officially
@@ -731,10 +801,19 @@ region_trace_ensemble(P7_DOMAINDEF *ddef, const P7_OPROFILE *om, const ESL_DSQ *
  *         for the domain upon return, but we're not making that
  *         part of the spec, so caller shouldn't rely on this;
  *         spec just makes its contents "undefined".
+ *         
+ * 
+ * Returns <eslFAIL> if domain is not successfully identified.  This
+ * is rare; one way it can happen is if posterior decoding calculation
+ * overflows, which can occur on highly repetitive sequence
+ * {J3/119-121}. Beware: as a result, it is possible to have
+ * <ddef->ndom = 0>, for nonzero region(s)/envelope(s). See {iss131}.
+ * 
  */
 static int
-rescore_isolated_domain(P7_DOMAINDEF *ddef, P7_DOMAINDEF *ddef_app, const P7_OPROFILE *om, const ESL_SQ *sq,
-			P7_OMX *ox1, P7_OMX *ox2, int i, int j, int null2_is_done, P7_BG *bg, int long_target)
+rescore_isolated_domain(P7_DOMAINDEF *ddef, P7_OPROFILE *om, const ESL_SQ *sq, const ESL_SQ *ntsq,
+			P7_OMX *ox1, P7_OMX *ox2, int i, int j, int null2_is_done, P7_BG *bg, int long_target,
+			P7_BG *bg_tmp, float *scores_arr, float *fwd_emissions_arr)
 {
   P7_DOMAIN     *dom           = NULL;
   int            Ld            = j-i+1;
@@ -744,27 +823,38 @@ rescore_isolated_domain(P7_DOMAINDEF *ddef, P7_DOMAINDEF *ddef_app, const P7_OPR
   int            pos;
   float          null2[p7_MAXCODE];
   int            status;
-  float          null3_corr;
+  int            max_env_extra = 20;
+  int            orig_L;
+
+
+  if (long_target) {
+    //temporarily change model length to env_len. The nhmmer pipeline will tack
+    //on the appropriate cost to account for the longer actual window
+    orig_L = om->L;
+    p7_oprofile_ReconfigRestLength(om, j-i+1);
+  }
+
+  if (long_target && scores_arr!=NULL) {
+    // Modify bg and om in-place to avoid having to clone (allocate) a massive
+    // number of times when there are many hits
+    reparameterize_model (bg, om, sq, i, j-i+1, fwd_emissions_arr, bg_tmp->f, scores_arr);
+  }
 
   p7_Forward (sq->dsq + i-1, Ld, om,      ox1, &envsc);
   p7_Backward(sq->dsq + i-1, Ld, om, ox1, ox2, NULL);
 
-  if (ddef_app != NULL) {
-	  //grab mocc values for computing confidence of alignment-inclusion boundaries
-	  if ((status = p7_domaindef_GrowTo(ddef_app, Ld))      != eslOK) return status;  /* ddef_app btot,etot,mocc now ready*/
-	  if ((status = p7_DomainDecoding(om, ox1, ox2, ddef_app)) != eslOK && status != eslERANGE) return status;  /* ddef_app->{btot,etot,mocc} now made.                    */
-  }
-
-
   status = p7_Decoding(om, ox1, ox2, ox2);      /* <ox2> is now overwritten with post probabilities     */
-  if (status == eslERANGE) return eslFAIL;      /* rare: numeric overflow; domain is assumed to be repetitive garbage [J3/119-212] */
-
+  if (status == eslERANGE) { /* rare: numeric overflow; domain is assumed to be repetitive garbage [J3/119-121] */
+    if (long_target && scores_arr) 
+      reparameterize_model(bg, om, NULL, 0, 0, fwd_emissions_arr, bg_tmp->f, scores_arr); /* revert to original bg model */
+    status = eslFAIL;
+    goto ERROR;
+  }
 
   /* Find an optimal accuracy alignment */
   p7_OptimalAccuracy(om, ox2, ox1, &oasc);      /* <ox1> is now overwritten with OA scores              */
   p7_OATrace        (om, ox2, ox1, ddef->tr);   /* <tr>'s seq coords are offset by i-1, rel to orig dsq */
 
-
   /* hack the trace's sq coords to be correct w.r.t. original dsq */
   for (z = 0; z < ddef->tr->N; z++)
     if (ddef->tr->i[z] > 0) ddef->tr->i[z] += i-1;
@@ -772,65 +862,118 @@ rescore_isolated_domain(P7_DOMAINDEF *ddef, P7_DOMAINDEF *ddef_app, const P7_OPR
   /* get ptr to next empty domain structure in domaindef's results */
   if (ddef->ndom == ddef->nalloc) {
     ESL_REALLOC(ddef->dcl, sizeof(P7_DOMAIN) * (ddef->nalloc*2));
-    ddef->nalloc *= 2;    
+    ddef->nalloc *= 2;
   }
   dom = &(ddef->dcl[ddef->ndom]);
+  dom->ad             = p7_alidisplay_Create(ddef->tr, 0, om, sq, ntsq);
+  dom->scores_per_pos = NULL;
+
+
+  /* For long target DNA, it's common to see a huge envelope (>1Kb longer than alignment), usually
+   * involving simple repeat part of model that attracted similar segments of the repeatedly, to
+   * acquire a large total score. Now that we have alignment boundaries, re-run Fwd/Bkwd to trim away
+   * such a long envelope and estimate the true score of the hit region
+   */
+  if (long_target) {
+
+    if (     i < dom->ad->sqfrom-max_env_extra   //trim the left side of the envelope
+        ||   j > dom->ad->sqto+max_env_extra     //trim the right side of the envelope
+        ) {
+
+      //trim in the envelope, and do it again
+      i = ESL_MAX(i,dom->ad->sqfrom-max_env_extra);
+      j = ESL_MIN(j,dom->ad->sqto+max_env_extra);
+      Ld = j - i + 1;
+
+      //temporarily change model length to env_len. The nhmmer pipeline will tack
+      //on the appropriate cost to account for the longer actual window
+      p7_oprofile_ReconfigRestLength(om, j-i+1);
+
+      if (scores_arr!=NULL) {
+        //revert bg and om back to original, then forward to new values
+        reparameterize_model (bg, om, NULL, 0, 0, fwd_emissions_arr, bg_tmp->f, scores_arr);
+        reparameterize_model (bg, om, sq, i, Ld, fwd_emissions_arr, bg_tmp->f, scores_arr);
+      }
+
+      p7_Forward (sq->dsq + i-1, Ld, om,      ox1, &envsc);
+      p7_Backward(sq->dsq + i-1, Ld, om, ox1, ox2, NULL);
+
+      status = p7_Decoding(om, ox1, ox2, ox2);      /* <ox2> is now overwritten with post probabilities     */
+      if (status == eslERANGE) { /* rare: numeric overflow; domain is assumed to be repetitive garbage [J3/119-121] */
+          reparameterize_model(bg, om, NULL, 0, 0, fwd_emissions_arr, bg_tmp->f, scores_arr); /* revert to original bg model */
+          status = eslFAIL;
+          goto ERROR;
+      }
+
+      /* Find an optimal accuracy alignment */
+      p7_OptimalAccuracy(om, ox2, ox1, &oasc);      /* <ox1> is now overwritten with OA scores              */
+      p7_trace_Reuse(ddef->tr);
+      p7_OATrace        (om, ox2, ox1, ddef->tr);   /* <tr>'s seq coords are offset by i-1, rel to orig dsq */
+
+      /* re-hack the trace's sq coords to be correct w.r.t. original dsq */
+       for (z = 0; z < ddef->tr->N; z++)
+         if (ddef->tr->i[z] > 0) ddef->tr->i[z] += i-1;
+
+       /* store the results in it, first destroying the old alidisplay object */
+       p7_alidisplay_Destroy(dom->ad);
+       dom->ad            = p7_alidisplay_Create(ddef->tr, 0, om, sq, NULL);
+    }
+
+    /* Estimate bias correction, by computing what the score would've been without
+     * reparameterization
+     */
+    domcorrection = envsc;
+    if (scores_arr!=NULL) { //revert bg and om back to original,
+                            //and while I'm at it, capture what the default parameterized score would have been, for "null2"
+      reparameterize_model (bg, om, NULL, 0, 0, fwd_emissions_arr, bg_tmp->f, scores_arr);
+      p7_Forward (sq->dsq + i-1, Ld, om,      ox1, &domcorrection);
+    }
+
+    p7_oprofile_ReconfigRestLength(om, orig_L);
+
+    if (domcorrection < envsc)  //negative bias correction shouldn't happen. Stick with the original score.
+      envsc = domcorrection;
+
+    dom->domcorrection = domcorrection - envsc;
+
+  }  else {
+
+    /* Compute bias correction (for non-longtarget case)
+     *
+     * Is null2 set already for this i..j? (It is, if we're in a domain that
+     * was defined by stochastic traceback clustering in a multidomain region;
+     * it isn't yet, if we're in a simple one-domain region). If it isn't,
+     * do it now, by the expectation (posterior decoding) method.
+     */
+      if (!null2_is_done) {
+        p7_Null2_ByExpectation(om, ox2, null2);
+        for (pos = i; pos <= j; pos++)
+          ddef->n2sc[pos]  = logf(null2[sq->dsq[pos]]);
+      }
+      for (pos = i; pos <= j; pos++)
+        domcorrection   += ddef->n2sc[pos];         /* domcorrection is in units of NATS */
+
+      dom->domcorrection = domcorrection; /* in units of NATS */
+
+  }
+
 
-  /* store the results in it */
-  dom->ad            = p7_alidisplay_Create(ddef->tr, 0, om, sq, ddef_app);
   dom->iali          = dom->ad->sqfrom;
   dom->jali          = dom->ad->sqto;
-  dom->ihq           = dom->ad->hqfrom;
-  dom->jhq           = dom->ad->hqto;
   dom->ienv          = i;
   dom->jenv          = j;
   dom->envsc         = envsc;         /* in units of NATS */
-  dom->oasc          = oasc;	      /* in units of expected # of correctly aligned residues */
-  dom->dombias       = 0.0;	/* gets set later, using bg->omega and dombias */
-  dom->bitscore      = 0.0;	/* gets set later by caller, using envsc, null score, and dombias */
-  dom->lnP           = 0.0;	/* gets set later by caller, using bitscore */
-  dom->is_reported   = FALSE;	/* gets set later by caller */
-  dom->is_included   = FALSE;	/* gets set later by caller */
+  dom->oasc          = oasc;        /* in units of expected # of correctly aligned residues */
+  dom->dombias       = 0.0; /* gets set later, using bg->omega and dombias */
+  dom->bitscore      = 0.0; /* gets set later by caller, using envsc, null score, and dombias */
+  dom->lnP           = 0.0; /* gets set later by caller, using bitscore */
+  dom->is_reported   = FALSE; /* gets set later by caller */
+  dom->is_included   = FALSE; /* gets set later by caller */
 
 
-
-  /* Compute bias correction
-   *
-   * Is null2 set already for this i..j? (It is, if we're in a domain that
-   * was defined by stochastic traceback clustering in a multidomain region;
-   * it isn't yet, if we're in a simple one-domain region). If it isn't,
-   * do it now, by the expectation (posterior decoding) method.
-   */
-  if (long_target || !null2_is_done) {
-    p7_Null2_ByExpectation(om, ox2, null2);
-    for (pos = i; pos <= j; pos++)
-      ddef->n2sc[pos]  = logf(null2[sq->dsq[pos]]);
-  }
-  for (pos = i; pos <= j; pos++)
-    domcorrection   += ddef->n2sc[pos];         /* domcorrection is in units of NATS */
-
-  if (long_target) {
-    /* for long_target case, merge in the results of two null3 correction
-     * see ~/notebook/2012/0214_Dfam_false_positives/00NOTES 02/19 for details
-     *
-     * bias composition is based only on the aligned positions:
-     */
-    if (bg->use_null3) {
-      p7_null3_score(om->abc, sq->dsq, NULL /*don't use trace*/, dom->iali, dom->jali, bg, &null3_corr);
-      domcorrection = p7_FLogsum (domcorrection, null3_corr);
-    }
-    if (bg->use_null3w) {
-      p7_null3_windowed_score(om->abc, sq->dsq, dom->iali, dom->jali, bg, &null3_corr);
-      domcorrection = p7_FLogsum (domcorrection, null3_corr);
-    }
-  }
-
-  dom->domcorrection = domcorrection; /* in units of NATS */
-
   ddef->ndom++;
 
   p7_trace_Reuse(ddef->tr);
-  if (ddef_app != NULL) p7_domaindef_Reuse(ddef_app);
   return eslOK;
 
  ERROR:
@@ -942,14 +1085,16 @@ main(int argc, char **argv)
 
   p7_Forward (sq->dsq, sq->n, om,      fwd, &overall_sc); 
   p7_Backward(sq->dsq, sq->n, om, fwd, bck, &sc);       
-  p7_domaindef_ByPosteriorHeuristics(sq, om, oxf, oxb, fwd, bck, ddef, NULL, NULL, FALSE);
+  p7_domaindef_ByPosteriorHeuristics(sq, NULL, om, oxf, oxb, fwd, bck, ddef, NULL, FALSE, NULL, NULL, NULL);
+
 
   printf("Overall raw likelihood score: %.2f nats\n", overall_sc);
 
   /* retrieve and display results */
   for (d = 0; d < ddef->ndom; d++)
     {
-      printf("domain %-4d : %4d %4d  %6.2f  %6.2f\n", d+1, 
+      printf("domain %-4d : %4" PRId64 " %4" PRId64 "  %6.2f  %6.2f\n", 
+	     d+1, 
 	     ddef->dcl[d].ienv,
 	     ddef->dcl[d].jenv,
 	     ddef->dcl[d].envsc,
@@ -1029,6 +1174,9 @@ main(int argc, char **argv)
   P7_TRACE       *tr      = NULL;
   P7_GMX         *fwd     = NULL;
   P7_GMX         *bck     = NULL;
+  P7_OMX         *gxf     = NULL; /* parsing matrix, passed to PosteriorHeuristics */
+  P7_OMX         *gxb     = NULL;
+
   P7_DOMAINDEF   *ddef    = NULL;
   int   N           = esl_opt_GetInteger(go, "-N");
   int   L0          = esl_opt_GetInteger(go, "-L");
@@ -1055,6 +1203,8 @@ main(int argc, char **argv)
   ddef = p7_domaindef_Create(r);
   fwd  = p7_gmx_Create(gm->M, L0);
   bck  = p7_gmx_Create(gm->M, L0);
+  gxf  = p7_gmx_Create(gm->M, L0);
+  gxb  = p7_gmx_Create(gm->M, L0);
   tr   = p7_trace_Create();
   p7_FLogsumInit();
 
@@ -1084,7 +1234,8 @@ main(int argc, char **argv)
 	  p7_GForward (sq->dsq, sq->n, gm, fwd, &overall_sc); 
 	  if (! do_baseline) {
 	    p7_GBackward(sq->dsq, sq->n, gm, bck, &sc);       
-	    p7_domaindef_ByPosteriorHeuristics(gm, sq, fwd, bck, ddef, NULL,  NULL, FALSE);
+	    p7_domaindef_ByPosteriorHeuristics(sq, gm, fwd, bck, gxf, gxb, ddef, NULL, FALSE, NULL, NULL, NULL);
+	    //Is this even being compiled by any tests? Looks like there's a fair amount of bit rot here
 	  }
 	}
 
