@@ -8,8 +8,6 @@
  *   5. Other routines in the API.
  *   6. Unit tests.
  *   7. Test driver. 
- *   8. Copyright and license.
- * 
  */
 #include "p7_config.h"
 
@@ -823,15 +821,12 @@ int
 p7_hmm_ScaleExponential(P7_HMM *hmm, double exp)
 {
 
-//  printf ("\n==================\nTry exp = %.4f\n", exp);
   int k;
   for (k = 1; k <= hmm->M; k++) {
 
     float count = esl_vec_FSum(hmm->mat[k], hmm->abc->K);
     float new_count = pow(count, exp);
-    double scale = new_count / count;
-
-    //printf ("column %d : cnt: %.1f,  new_cnt: %.1f,  scale: %.4f\n", k, count, new_count, scale );
+    double scale = count>0 ? new_count / count : 1.0;  /* if no counts in the column (strange, but possible), just use default freqs*/
 
     esl_vec_FScale(hmm->t[k],   p7H_NTRANSITIONS, scale);
     esl_vec_FScale(hmm->mat[k], hmm->abc->K,      scale);
@@ -1497,16 +1492,3 @@ main(int argc, char **argv)
 /*-------------------- end of test driver ---------------------*/
 
 
-/************************************************************
- * HMMER - Biological sequence analysis with profile HMMs
- * Version 3.1b3; July 2016
- * Copyright (C) 2016 Howard Hughes Medical Institute.
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
- * 
- * HMMER is distributed under the terms of the GNU General Public License
- * (GPLv3). See the LICENSE file for details.
- *
- * SVN $Id$
- * SVN $URL$
- ************************************************************/
-
